

local f = CreateFrame ("frame")
f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("PLAYER_TARGET_CHANGED")
local select = select
local strsplit = strsplit
local UnitGUID = UnitGUID
local UnitIsDead = UnitIsDead
local playerName, lastSourceName, lastSourceGUID, amount, overkill, spellId, spellName, combatEventRegistered, verifiedKill
local UnregisterCombatEvent = function()
    f:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    verifiedKill = true
    combatEventRegistered = false
end
local RegisterCombatEvent = function()
    f:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    verifiedKill = false
    combatEventRegistered = true
end

local logType = {
    ["RANGE_DAMAGE"] = true,
    ["SPELL_DAMAGE"] = true,
    ["SPELL_PERIODIC_DAMAGE"] = true,
    ["SPELL_BUILDING_DAMAGE"] = true
}
local NPCData = {
    ["32491"] = true, -- Time-Lost Proto-Drake
    ["50062"] = true, -- Aeonaxx
    ["50005"] = true, -- Poseidus
    ["50981"] = true, -- Luk'hok
    ["50992"] = true, -- Gorok
    ["50883"] = true, -- Pathrunner
    ["50985"] = true, -- Poundfist
    ["50990"] = true -- Nakk the Thunderer
}

local function formatUnit(guid, name, classname)
	local classColors = _G.RAID_CLASS_COLORS
    if classColors then
		local color = classColors[classname]
		if (type(name) == "string" and color) then
            local cname = "|c" .. color.colorStr .. name .. "|r"
			return ("|Hunit:"..guid..":"..name.."|h"..cname.."|h")
		end
	end
	return ("|Hunit:"..guid..":"..name.."|h"..name.."|h")
end

local function GetClass(GUID, name)
    if name == "Unknown" then return end
    local class
    local _, classname = UnitClass(name)
    if classname then
        class = classname
    else
        local classname = select(2,GetPlayerInfoByGUID(GUID))
        if classname then
            class = classname
        end
    end
    return class
end

f:SetScript ("OnEvent", function (self, event, ...)
    if event == "PLAYER_LOGIN" then
        playerName = GetUnitName("player")
    elseif event == "PLAYER_TARGET_CHANGED" then
        local GUID = UnitGUID("target")
        if not GUID then return end
        local npcID=select(6,strsplit("-", GUID))
        if NPCData[npcID] then
            if UnitIsDead("target") then return end
            if not combatEventRegistered then
                RegisterCombatEvent()
            end
        end
    elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
        local _, type, _, sourceGUID, sourceName, _, _, destGUID, destName = CombatLogGetCurrentEventInfo()
        if not destGUID then return end
        if logType[type] then
            local npcID=select(6,strsplit("-", destGUID))
            if NPCData[npcID] and playerName ~= sourceName then
                spellId, spellName, _, amount, overkill  = select(12, CombatLogGetCurrentEventInfo())
                local class = GetClass(sourceGUID, sourceName)
                lastSourceName = sourceName
                lastSourceGUID = sourceGUID
                if overkill > 1 then
                    print("|cFFF9FF09"..formatUnit(destGUID, destName).." killed by |r"..formatUnit(sourceGUID, sourceName, class).."|r|cFFF9FF09 with|r |cFF83E9E9"..(GetSpellLink(spellId) or spellName).."|r|cFFF9FF09, Damage: "..amount.." ("..overkill.." Overkill)")
                    UnregisterCombatEvent()
                end
            end
        elseif type == "SWING_DAMAGE" then
            local npcID=select(6,strsplit("-", destGUID))
            if NPCData[npcID] and playerName ~= sourceName then
                amount, overkill  = select(12, CombatLogGetCurrentEventInfo())
                local class = GetClass(sourceGUID, sourceName)
                lastSourceName = sourceName
                lastSourceGUID = sourceGUID
                if overkill > 1 then
                    print("|cFFF9FF09"..formatUnit(destGUID, destName).." killed by |r|cFFEC5357"..formatUnit(sourceGUID, sourceName, class).."|r|cFFF9FF09 with|r |cFF83E9E9Melee|r|cFFF9FF09, Damage: "..amount.." ("..overkill.." Overkill)")
                    UnregisterCombatEvent()
                end
            end
        elseif type == "PARTY_KILL" then
            if playerName == sourceName then
                print("|cFFF9FF09You killed "..formatUnit(destGUID, destName).."|r")
                UnregisterCombatEvent()
            end
        elseif type == "UNIT_DIED" then
            local npcID=select(6,strsplit("-", destGUID))
            if NPCData[npcID] and playerName ~= lastSourceName then
                if not verifiedKill then
                    lastSourceName = lastSourceName or "Unknown"
                    lastSourceGUID = sourceGUID or ""
                    amount = amount or "Unknown"
                    overkill = overkill or "Unknown"
                    local class = GetClass(lastSourceGUID, lastSourceName)
                    print("|cFFF9FF09"..formatUnit(destGUID, destName).." might have been killed by |r|cFFEC5357"..formatUnit(lastSourceGUID, lastSourceName, class).."|r|cFFF9FF09, Damage: "..amount.." ("..overkill.." Overkill)")
                    print("|cFFF9FF09Double Check in Combat Log.|r")
                    UnregisterCombatEvent()
                end
            end
        end
    end
end)